import 'package:fl_assets_picker/fl_assets_picker.dart';
import 'package:flutter/material.dart';

class AssetsPickerEntryBuild extends StatelessWidget {
  const AssetsPickerEntryBuild(this.entry,
      {super.key, this.isThumbnail = true, required this.fit});

  final ExtendedAssetEntity entry;

  /// 是否优先预览缩略图
  final bool isThumbnail;
  final BoxFit fit;

  @override
  Widget build(BuildContext context) {
    switch (entry.type) {
      case AssetType.other:
        return _AssetEntryOtherBuild(entry, isThumbnail: isThumbnail, fit: fit);
      case AssetType.image:
        return _AssetEntryImageBuild(entry, isThumbnail: isThumbnail, fit: fit);
      case AssetType.video:
        return _AssetEntryVideoBuild(entry, isThumbnail: isThumbnail, fit: fit);
      case AssetType.audio:
        return _AssetEntryOtherBuild(entry, isThumbnail: isThumbnail, fit: fit);
    }
  }
}

class _AssetEntryVideoBuild extends StatelessWidget {
  const _AssetEntryVideoBuild(this.asset,
      {this.isThumbnail = false, required this.fit});

  final ExtendedAssetEntity asset;
  final bool isThumbnail;
  final BoxFit fit;

  @override
  Widget build(BuildContext context) {
    if (asset.type == AssetType.video) {
      Widget? current;
      ImageProvider? imageProvider;
      if (asset.thumbnailDataAsync != null) {
        imageProvider = ExtendedMemoryImageProvider(asset.thumbnailDataAsync!);
      } else if (asset.renovated != null) {
        imageProvider =
            ExtendedImageWithAssetsPicker.buildImageProvider(asset.renovated);
      }
      if (isThumbnail && imageProvider != null) {
        current = ExtendedImageWithAssetsPicker(imageProvider, fit: fit);
      } else {
        final controller =
            FlVideoPlayerWithAssetsPicker.buildVideoPlayerController(
                asset.previewed ?? asset.fileAsync);
        if (controller != null) {
          current = FlVideoPlayerWithAssetsPicker(
              controller: controller, cover: current);
        }
      }
      if (current != null) return current;
    }
    return const _PreviewSupported();
  }
}

class _AssetEntryImageBuild extends StatelessWidget {
  const _AssetEntryImageBuild(this.asset,
      {this.isThumbnail = false, required this.fit});

  final ExtendedAssetEntity asset;
  final bool isThumbnail;
  final BoxFit fit;

  @override
  Widget build(BuildContext context) {
    if (asset.type == AssetType.image) {
      ImageProvider? imageProvider;
      if (asset.thumbnailDataAsync != null) {
        imageProvider =
            ExtendedImageWithAssetsPicker.assetEntityToImageProvider(asset);
      }
      imageProvider = isThumbnail && imageProvider != null
          ? imageProvider
          : ExtendedImageWithAssetsPicker.assetEntityToImageProvider(asset);
      if (imageProvider == null) return const SizedBox();
      return ExtendedImageWithAssetsPicker(imageProvider, fit: fit);
    }
    return const _PreviewSupported();
  }
}

class _AssetEntryOtherBuild extends StatelessWidget {
  const _AssetEntryOtherBuild(this.asset,
      {this.isThumbnail = false, required this.fit});

  final ExtendedAssetEntity asset;
  final bool isThumbnail;
  final BoxFit fit;

  @override
  Widget build(BuildContext context) {
    if (asset.type == AssetType.other || asset.type == AssetType.audio) {
      ImageProvider? thumbnailProvider;
      if (asset.thumbnailDataAsync != null) {
        thumbnailProvider =
            ExtendedImageWithAssetsPicker.assetEntityToImageProvider(asset);
      }
      if (isThumbnail && thumbnailProvider != null) {
        return ExtendedImageWithAssetsPicker(thumbnailProvider, fit: fit);
      }
    }
    return const _PreviewSupported();
  }
}

class _PreviewSupported extends StatelessWidget {
  const _PreviewSupported();

  @override
  Widget build(BuildContext context) =>
      const Center(child: Text('Preview not supported'));
}
